# COPYRIGHT 2022 MONOOCLE INSIGHTS
# ALL RIGHTS RESERVED
# UNAUTHORIZED USE, REPRODUCTION, OR
# DISTRIBUTION STRICTLY PROHIBITED
#
# AUTHOR: Raymond Deiotte
# CREATION DATE: <DATE>
# LAST UPDATED: <DATE>


import pandas as pd


class ThisClass:

    def __init__(self):
        '''
        Method to intialize the class
        '''

    def doIt(self):
        '''
        Method to control the class
        '''


if __name__ == '__main__':
    c = ThisClass()

    c.doIt()
